
package org.daml.rdf.jena.mem;

import org.daml.rdf.jena.model.SourceModel;
import org.daml.rdf.jena.model.Source;
import org.daml.rdf.jena.common.SourceStore;
import com.hp.hpl.mesa.rdf.jena.common.ModelI;
import com.hp.hpl.mesa.rdf.jena.mem.ModelMem;
import com.hp.hpl.mesa.rdf.jena.model.Model;
import com.hp.hpl.mesa.rdf.jena.model.RDFException;
import com.hp.hpl.mesa.rdf.jena.model.Statement;
import com.hp.hpl.mesa.rdf.jena.model.StmtIterator;
import com.hp.hpl.mesa.rdf.jena.common.StatementImpl;
import com.hp.hpl.mesa.rdf.jena.common.StmtIteratorImpl;
import java.util.Iterator;
import java.io.Reader;

/**
 * An implementation of a RDF Model that keeps statements grouped by source.
 *
 * @author der
 * @version $Id: SourceModelMem.java,v 1.2 2003/03/26 22:39:19 drager Exp $
 */
public class SourceModelMem extends ModelMem implements SourceModel, ModelI
{

    // ================================================================
    // Constructor methods 
    // ================================================================

    public SourceModelMem() 
    {
	super();
	store = new SourceStoreMem();
    }

    public SourceModelMem(Source source)
    {
	super();
	store = new SourceStoreMem(source);
    }

    /**
     * Add all the statements in another model to this model and associate with a source.
     */
    public Model add(Model m, Source source)
	throws RDFException
    {
	return add(m.listStatements(), source);
    }

    /**
     * Add a statement to this model and associate with a source.
     */
    public Model add(Statement s, Source source)
	throws RDFException
    {
	if (! (s instanceof StatementImpl) ||
	    (((StatementImpl)s).getModel() != this) ) {
	    s = new StatementImpl(s.getSubject(),
				  s.getPredicate(),
				  s.getObject(),
				  this);
	} 
	((SourceStore)store).add(s, source);
	return this;
    }

    /**
     * Add all the statements in an interator to this model and associate with a source.
     */
    public Model add(StmtIterator iter, Source source)
	throws RDFException
    {
	try {
	    while (iter.hasNext()) {
		add(iter.next(), source);
	    }
	    return this;
	} finally {
	    iter.close();
	}
    }

    /**
     * List all sources with statements in the model.
     */
    public Iterator listSources()
    {
	return ((SourceStore)store).listSources(); 
    }
    
    /**
     * Return an iterator over all the statements associated with a source.
     */
    public StmtIterator listStatementsOfSource(Source source)
    {
	return new StmtIteratorImpl(((SourceStore)store).listBySource(source), null);
    }

    /**
     * Return a model that contains all the statements associated with a source.
     * TBD - Allow user to specify what type of model to return.
     */
    public Model getModelForSource(Source source)
	throws RDFException
    {
	SourceModelMem model = new SourceModelMem();
	model.add(listStatementsOfSource(source), source);
	return (Model)model;
    }

    /**
     * Change the source associated with a statement.
     */
    public Model setSource(Statement s, Source source)
    {
	((SourceStore)store).setSource(s, source);
	return this;
    }

    /**
     * Change the source associated with the statements in the iterator.
     */
    public Model setSource(StmtIterator iter, Source source)
	throws RDFException
    {
	try {
	    while (iter.hasNext()) {
		((SourceStore)store).setSource(iter.next(), source);
	    }
	    return this;
	} finally {
	    iter.close();
	}
    }

    /**
     * Write the model of statements associated with a source to an XML file.
     */
    public Model write(java.io.Writer writer, Source source) 
	throws RDFException
    {
	Model model = getModelForSource(source);
	model.write(writer);
	return this;
    }
    public Model write(java.io.Writer writer, java.lang.String lang, Source source)
	throws RDFException
    {
	Model model = getModelForSource(source);
	model.write(writer, lang);
	return this;
    }

    public Model write(java.io.Writer writer, java.lang.String lang, java.lang.String base, Source source)
	throws RDFException
    {
	Model model = getModelForSource(source);
	model.write(writer, lang, base);
	return this;
    }

    // ================================================================
    // Override methods from ModelMem
    // ================================================================

    /**
     * Read a URL into the model.
     * The Source for the statements will be the url.
     */
    public Model read(String url) 
	throws RDFException 
    {
	SourceStore sStore = (SourceStore)store;
	Source originalSource = sStore.getDefaultSource();
	Source newSource = sStore.createSource(url);
	try {
	    sStore.setDefaultSource(newSource);
	    super.read(url);
	    return this;
	} finally {
	    sStore.setDefaultSource(originalSource);
	}
    }

    /**
     * Read RDF from a reader into the model.
     * The Source for the statements will be the base uri.
     */
    public Model read(Reader reader, String base, Source source) 
	throws RDFException
    {
	SourceStore sStore = (SourceStore)store;
	Source originalSource = sStore.getDefaultSource();
	try {
	    sStore.setDefaultSource(source);
	    super.read(reader, base);
	    return this;
	} finally {
	    sStore.setDefaultSource(originalSource);
	}
    }

    /**
     * Read a URL into the model.
     * The Source for the statements will be the url.
     */
    public Model read(String url, String lang) 
	throws RDFException
    {
	SourceStore sStore = (SourceStore)store;
	Source originalSource = sStore.getDefaultSource();
	Source newSource = sStore.createSource(url);
	try {
	    sStore.setDefaultSource(newSource);
	    super.read(url, lang);
	    return this;
	} finally {
	    sStore.setDefaultSource(originalSource);
	}
    }

    /**
     * Read RDF from a reader into the model.
     * The Source for the statements will be the base uri.
     */
    public Model read(Reader reader, String base, String lang, Source source)
	throws RDFException
    {
	SourceStore sStore = (SourceStore)store;
	Source originalSource = sStore.getDefaultSource();
	try {
	    sStore.setDefaultSource(source);
	    super.read(reader, base, lang);
	    return this;
	} finally {
	    sStore.setDefaultSource(originalSource);
	}
    }

    public com.hp.hpl.mesa.rdf.jena.model.Resource createResource(String uri)
	throws RDFException
    {
	//System.out.println("**** IN CreateResource");
	return super.createResource(uri);
    }

    public Source setDefaultSource(Source source)
    {
	Source old = ((SourceStore)store).getDefaultSource();
	((SourceStore)store).setDefaultSource(source);
	return old;
    }

    public Source getDefaultSource()
    {
	return ((SourceStore)store).getDefaultSource();
    }

    public Source createSource(String uri)
    {
	return ((SourceStore)store).createSource(uri);
    }
}
