package org.daml.rdf.jena.mem;

import org.daml.rdf.jena.model.Source;
import org.daml.rdf.jena.common.SourceStore;
import org.daml.rdf.jena.common.SourceImpl;
import org.daml.rdf.jena.mem.StoreMem;
import com.hp.hpl.mesa.rdf.jena.common.Store;
import com.hp.hpl.mesa.rdf.jena.model.*;
//import com.hp.hpl.mesa.rdf.jena.mem.StoreMem;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.HashMap;
import java.util.Set;


/**
 * Implementation of a memory based triple store that keeps track of the source for each statement.
 *
 * @author drager
 * @version $Id: SourceStoreMem.java,v 1.1.1.1 2003/03/26 20:35:40 drager Exp $
 */
public class SourceStoreMem extends StoreMem implements SourceStore
{
    // Mapping from a source to the statements associated with it.
    SourceMap sourceToStatements = new SourceMap();

    // Mapping from a statement to it's source.
    // Do we want to store source in the statement object?
    HashMap statementToSource = new HashMap();

    Source defaultSource = null;

    // ================================================================
    // Constructors
    // ================================================================

    public SourceStoreMem(Source source) 
    {
	defaultSource = source;
    }

    public SourceStoreMem()
    {
    }

    // ================================================================
    // Implementations of SourceStore specific methods
    // ================================================================

    /**
     * Add a statement associated with a source.
     * TBD: What to do if statement already belongs to store, but with a different source?
     */
    public synchronized void add(Statement s, Source source) 
	throws RDFException
    {
	if (statements.contains(s)) return;
	statementToSource.put(s, source);
	sourceToStatements.add(source, s);
	super.add(s);
    }

    /**
     * Return an iterator over all the statements with a given source.
     */
    public synchronized Iterator listBySource(Source source) 
    {
	return sourceToStatements.iterator(source);
    }

    /**
     * Return the store of a statement.
     */
    public synchronized Source getSource(Statement s) 
    {
	return (Source)statementToSource.get(s);
    }

    /**
     * Return an iterator over all the sources in the store.
     */
    public synchronized Iterator listSources() 
    {
	return sourceToStatements.keySet().iterator();
    }

    /**
     * Set the default source that is used when adding a statement.
     */
    public synchronized void setDefaultSource(Source source)
    {
	defaultSource = source;
    }

    /**
     * Get the default source that is used when adding a statement.
     */
    public synchronized Source getDefaultSource()
    {
	return defaultSource;
    }

    /**
     * Change the source of a statement.
     */
    public synchronized void setSource(Statement s, Source source)
    {
	if (statements.contains(s)) {
	    //originalSource = getSource(s);
	    statementToSource.put(s, source);
	}
    }

    /**
     * Create a new source.
     */
    public synchronized Source createSource(String uri)
    {
	return new SourceImpl(uri);
    }


    // ================================================================
    // Implementation of methods specific to Source
    // ================================================================

    public void add(Statement s)
	throws RDFException
    {	
	add(s, defaultSource);
    }

    public void remove(Statement s)
	throws RDFException
    {
	Source source = getSource(s);
	sourceToStatements.remove(source, s);
	super.remove(s);
    }


    // ================================================================

    protected class SourceMap 
    {
	HashMap map = new HashMap();
	
	protected void add(Source source, Statement s) {
	    LinkedList l = (LinkedList) map.get(source);
	    if (l == null) {
		l = new LinkedList();
		map.put(source, l);
	    }
	    l.add(s);
	}

	protected void remove(Source source, Statement s) {
	    LinkedList l = (LinkedList) map.get(source);
	    if (l != null) {
		l.remove(s);
		if (l.size() == 0) {
		    map.put(source, null);
		}
	    }
	}
    
	protected Iterator iterator(Source source)
	{
	    LinkedList l = (LinkedList) map.get(source);
	    if (l == null) {
		return (new LinkedList()).iterator();
	    } else {
		return l.iterator();
	    }
	}

	protected Set keySet()
	{
	    return map.keySet();
	}

    }
    // ================================================================
}

